@@ -1,22 +1,34 @@
 # coding: utf-8
 # Standard Python libraries
+import io
 from pathlib import Path
-import json
-
-from DataModelDict import DataModelDict as DM
+from typing import Callable, Optional, Tuple, Union
 
+# https://numpy.org/
 import numpy as np
+
+# https://pandas.pydata.org/
 import pandas as pd
 
-from datamodelbase import load_record
+# https://github.com/usnistgov/DataModelDict
+from DataModelDict import DataModelDict as DM
 
-from ..tools import aslist
+# https://github.com/usnistgov/yabadaba
+from yabadaba.record import Record
+from yabadaba import load_record
 
-def get_records(self, style=None, name=None,
-                local=None, remote=None, verbose=False,
-                refresh_cache=False, return_df=False, **kwargs):
+def get_records(self,
+                style: Optional[str] = None,
+                name: Union[str, list, None] = None,
+                local: Optional[bool] = None,
+                remote: Optional[bool] = None,
+                refresh_cache: bool = False,
+                return_df: bool = False,
+                verbose: bool = False,
+                **kwargs
+                ) -> Union[np.ndarray, Tuple[np.ndarray, pd.DataFrame]]:
     """
-    Retrieves all matching records from the local and/or remote locations.  If
+    Gets all matching records from the local and/or remote locations.  If
     records with the same record name are retrieved from both locations, then
     the local versions of those records are given.
 
@@ -32,9 +44,6 @@ def get_records(self, style=None, name=None,
     remote : bool, optional
         Indicates if the remote location is to be searched.  Default value
         matches the value set when the database was initialized.
-    verbose : bool, optional
-        If True, info messages will be printed during operations.  Default
-        value is False.
     refresh_cache : bool, optional
         If the local database is of style "local", indicates if the metadata
         cache file is to be refreshed.  If False,
@@ -45,6 +54,9 @@ def get_records(self, style=None, name=None,
     return_df : bool, optional
         If True, then the corresponding pandas.Dataframe of metadata
         will also be returned.
+    verbose : bool, optional
+        If True, info messages will be printed during operations.  Default
+        value is False.
     **kwargs : any, optional
         Any extra keyword arguments supported by the record style.
 
@@ -131,12 +143,18 @@ def get_records(self, style=None, name=None,
     else:
         return records
     
-def get_record(self, style=None, name=None,
-               local=None, remote=None,
-               prompt=True, promptfxn=None,
-               verbose=False, refresh_cache=False, **kwargs):
+def get_record(self, 
+               style: Optional[str] = None,
+               name: Union[str, list, None] = None,
+               local: Optional[bool] = None,
+               remote: Optional[bool] = None, 
+               prompt: bool = True,
+               promptfxn: Optional[Callable] = None,
+               refresh_cache: bool = False,
+               verbose: bool = False,
+               **kwargs) -> Record:
     """
-    Retrieves a single matching record from the local and/or remote locations.
+    Gets a single matching record from the local and/or remote locations.
     If local is True and the record is found there, then the local copy of the
     record is returned without searching the remote.
 
@@ -159,9 +177,6 @@ def get_record(self, style=None, name=None,
     promptfxn : function, optional
         A function that generates the prompt selection list.  If not given,
         the prompt will be a list of "id" values. 
-    verbose : bool, optional
-        If True, info messages will be printed during operations.  Default
-        value is False.
     refresh_cache : bool, optional
         If the local database is of style "local", indicates if the metadata
         cache file is to be refreshed.  If False,
@@ -169,6 +184,9 @@ def get_record(self, style=None, name=None,
         fields will not be updated.  If True, then the metadata for all
         records will be regenerated, which is needed to update the metadata
         for modified records.
+    verbose : bool, optional
+        If True, info messages will be printed during operations.  Default
+        value is False.
     **kwargs : any, optional
         Any extra keyword arguments supported by the record style.
 
@@ -251,8 +269,110 @@ def get_record(self, style=None, name=None,
     
     raise ValueError('No matching records found')
 
-def remote_query(self, style=None, keyword=None, query=None, name=None,
-                 return_df=False):
+def retrieve_record(self, 
+                    style: Optional[str] = None,
+                    name: Union[str, list, None] = None,
+                    dest: Optional[Path] = None,
+                    local: Optional[bool] = None,
+                    remote: Optional[bool] = None, 
+                    prompt: bool = True,
+                    promptfxn: Optional[Callable] = None,
+                    format: str = 'json',
+                    indent: int = 4, 
+                    refresh_cache: bool = False,
+                    verbose: bool = False,
+                    **kwargs):
+    """
+    Gets a single matching record from the database and saves it to a
+    file based on the record's name.
+
+    Parameters
+    ----------
+    style : str, optional
+        The record style to search. If not given, a prompt will ask for it.
+    name : str or list, optional
+        The name(s) of records to limit the search by.
+    dest : path, optional
+        The parent directory where the record will be saved to.  If not given,
+        will use the current working directory.
+    local : bool, optional
+        Indicates if the local location is to be searched.  Default value
+        matches the value set when the database was initialized.
+    remote : bool, optional
+        Indicates if the remote location is to be searched.  Default value
+        matches the value set when the database was initialized.
+    prompt : bool, optional
+        If prompt=True (default) then a screen input will ask for a selection
+        if multiple matching potentials are found.  If prompt=False, then an
+        error will be thrown if multiple matches are found.
+    promptfxn : function, optional
+        A function that generates the prompt selection list.  If not given,
+        the prompt will be a list of "id" values.
+    format : str, optional
+        The file format to save the record in: 'json' or 'xml'.  Default
+        is 'json'.
+    indent : int, optional
+        The number of space indentation spacings to use in the saved
+        record for the different tiered levels.  Default is 4.  Giving None
+        will create a compact record.
+    refresh_cache : bool, optional
+        If the local database is of style "local", indicates if the metadata
+        cache file is to be refreshed.  If False,
+        metadata for new records will be added but the old record metadata
+        fields will not be updated.  If True, then the metadata for all
+        records will be regenerated, which is needed to update the metadata
+        for modified records.
+    verbose : bool, optional
+        If True, info messages will be printed during operations.  Default
+        value is False.
+    **kwargs : any, optional
+        Any extra keyword arguments supported by the record style.
+
+    
+    Raises
+    ------
+    ValueError
+        If local or remote is set to True when the corresponding database
+        interaction has not been set.
+    ValueError
+        If multiple or no matching records are discovered.
+    """
+    # Set default dest
+    if dest is None:
+        dest = Path.cwd()
+
+    # Get the record
+    record = self.get_record(
+        style=style, name=name, local=local, remote=remote, prompt=prompt,
+        promptfxn=promptfxn, verbose=verbose, refresh_cache=refresh_cache,
+        **kwargs)
+
+    # Save as json
+    if format == 'json':
+        fname = Path(dest, f'{record.name}.json')
+        with open(fname, 'w', encoding='UTF-8') as f:
+            record.model.json(fp=f, indent=indent, ensure_ascii=False)
+        if verbose:
+            print(f'{fname} saved')
+    
+    # Save as xml
+    elif format == 'xml':
+        fname = Path(dest, f'{record.name}.xml')
+        with open(fname, 'w', encoding='UTF-8') as f:
+            record.model.xml(fp=f, indent=indent)
+        if verbose:
+            print(f'{fname} saved')
+
+    else:
+        raise ValueError('Invalid format: must be json or xml.')
+
+def remote_query(self,
+                 style: Optional[str] = None,
+                 keyword: Optional[str] = None,
+                 query: Optional[dict] = None,
+                 name: Union[str, list, None] = None,
+                 return_df: bool = False
+                 )-> Union[np.ndarray, Tuple[np.ndarray, pd.DataFrame]]:
     """
     Allows for custom Mongo-style or keyword search queries to be performed on
     records from the remote database.
@@ -261,17 +381,17 @@ def remote_query(self, style=None, keyword=None, query=None, name=None,
     ----------
     style : str, optional
         The record style to search. If not given, a prompt will ask for it.
+    keyword : str, optional
+        Allows for a search of records whose contents contain a keyword.
+        Cannot be given with query.
+    query : dict, optional
+        A custom-built CDCS Mongo-style query to use for the record search.
+        Cannot be given with keyword.
     name : str or list, optional
         The name(s) of records to limit the search by.
     return_df : bool, optional
         If True, then the corresponding pandas.Dataframe of metadata
         will also be returned
-    query : dict, optional
-        A custom-built CDCS Mongo-style query to use for the record search.
-        Cannot be given with keyword.
-    keyword : str, optional
-        Allows for a search of records whose contents contain a keyword.
-        Cannot be given with query.
     
     Returns
     -------
@@ -283,9 +403,13 @@ def remote_query(self, style=None, keyword=None, query=None, name=None,
     return self.remote_database.get_records(style=style, return_df=return_df,
                                             query=query, keyword=keyword, name=name)
 
-def download_records(self, style=None, name=None, overwrite=False,
-                     keyword=None, query=None, return_records=False,
-                     verbose=False, **kwargs):
+def download_records(self,
+                     style: Optional[str] = None,
+                     name: Union[str, list, None] = None,
+                     overwrite: bool = False,
+                     return_records: bool = False,
+                     verbose: bool = False,
+                     **kwargs) -> Optional[np.ndarray]:
     """
     Retrieves all matching records from the remote location and saves them to
     the local location.
@@ -300,19 +424,12 @@ def download_records(self, style=None, name=None, overwrite=False,
         Flag indicating if any existing local records with names matching
         remote records are updated (True) or left unchanged (False).  Default
         value is False.
-    query : dict, optional
-        A custom-built CDCS-style query to use for the record search.
-        Alternative to passing in the record-specific metadata kwargs.
-        Note that name can be given with query.
-    keyword : str, optional
-        Allows for a search of records whose contents contain a keyword.
-        Alternative to giving query or kwargs.
-    verbose : bool, optional
-        If True, info messages will be printed during operations.  Default
-        value is False.
     return_records : bool, optional
         If True, the retrieved record objects are also returned.  Default
         value is False.
+    verbose : bool, optional
+        If True, info messages will be printed during operations.  Default
+        value is False.
     **kwargs : any, optional
         Any extra keyword arguments supported by the record style.
 
@@ -362,8 +479,13 @@ def download_records(self, style=None, name=None, overwrite=False,
     if return_records is True:
         return records
 
-def save_record(self, record=None, style=None, name=None,
-                  model=None, overwrite=False, verbose=False):
+def save_record(self,
+                record: Optional[Record] = None,
+                style: Optional[str] = None,
+                name: Optional[str] = None,
+                model: Union[str, io.IOBase, DM, None] = None,
+                overwrite: bool = False,
+                verbose: bool = False):
     """
     Saves a record to the local database.
     
@@ -392,15 +514,21 @@ def save_record(self, record=None, style=None, name=None,
     
     try:
         self.local_database.add_record(record=record, verbose=verbose)
-    except ValueError:
+    except ValueError as e:
         if overwrite:
             self.local_database.update_record(record=record,
                                               verbose=verbose)
         else:
-            raise ValueError('Matching record already exists: use overwrite=True to change it')
+            raise ValueError('Matching record already exists: use overwrite=True to change it') from e
 
-def upload_record(self, record=None, style=None, name=None,
-                  model=None, workspace=None, overwrite=False, verbose=False):
+def upload_record(self,
+                  record: Optional[Record] = None,
+                  style: Optional[str] = None,
+                  name: Optional[str] = None,
+                  model: Union[str, io.IOBase, DM, None] = None,
+                  workspace: Union[str, pd.Series, None] = None,
+                  overwrite: bool = False,
+                  verbose: bool = False):
     """
     Uploads a record to the remote database.  Requires an account for the remote
     location with write permissions.
@@ -417,9 +545,10 @@ def upload_record(self, record=None, style=None, name=None,
     name : str, optional
         The name to assign to the record.  Required if record is not given and
         model is not a file name.
-    workspace : str, optional
-        The workspace to assign the record to. If not given, no workspace will
-        be assigned (only accessible to user who submitted it).
+    workspace : str or pandas.Series, optional
+        The CDCS workspace or workspace name to assign the record to. If not
+        given, no workspace will be assigned (only accessible to user who
+        submitted it).
     overwrite : bool, optional
         Indicates what to do when a matching record is found in the remote
         location.  If False (default), then the record is not updated.  If
@@ -434,15 +563,20 @@ def upload_record(self, record=None, style=None, name=None,
     try:
         self.remote_database.add_record(record=record, workspace=workspace,
                                         verbose=verbose) 
-    except ValueError:
+    except ValueError as e:
         if overwrite:
             self.remote_database.update_record(record=record, workspace=workspace,
                                                verbose=verbose)
         else:
-            raise ValueError('Matching record already exists: use overwrite=True to change it')
+            raise ValueError('Matching record already exists: use overwrite=True to change it') from e
 
-def delete_record(self, record=None, style=None, name=None,
-                  local=True, remote=False, verbose=False):
+def delete_record(self,
+                  record: Optional[Record] = None,
+                  style: Optional[str] = None,
+                  name: Optional[str] = None,
+                  local: bool = True,
+                  remote: bool = False,
+                  verbose: bool = False):
     """
     Deletes a record from the local and/or remote locations.  
 
