@@ -4,5 +4,5 @@ from .ParamFileBuilder import ParamFileBuilder
 from .LibParamBuilder import LibParamBuilder
 from .EamBuilder import EamBuilder
 from .EimBuilder import EimBuilder
-#from .KimBuilder import KimBuilder
+from .KimBuilder import KimBuilder
 from .PairBuilder import PairBuilder
\ No newline at end of file
